from functools import partial
from six.moves.urllib.parse import urlunparse
from swagger_parser import SwaggerParser
from molotov.session import LoggedClientSession
from smwogger.util import get_content


class API(object):
    def __init__(self, path_or_url, verbose=False, loop=None,
                 stream=None):
        self._content = get_content(path_or_url)
        self._parser = SwaggerParser(swagger_dict=self._content)
        self.spec = self._parser.specification
        self.verbose = verbose
        self.host = self.spec['host']
        if 'basePath' in self.spec:
            self.host += self.spec['basePath']
        schemes = self.spec.get('schemes', ['https'])
        self.scheme = schemes[0]
        self.running = True
        self._operations = self._get_operations()
        self.session = LoggedClientSession(loop, stream, verbose=verbose)

    def close(self):
        self.session.close()
        self.running = False

    async def __aenter__(self):
        self.session.__aenter__()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        self.session.__aexit__(exc_type, exc_val, exc_tb)
        self.session.close()
        self.running = False

    def __enter__(self):
        self.session.__enter__()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.session.__exit__(exc_type, exc_val, exc_tb)
        self.session.close()
        self.running = False

    def __getattr__(self, name):
        if name == 'operations':
            ops = list(self._get_operations().keys())
            ops.sort()
            return ops

        if name in self._operations:
            return partial(self._caller, name)
        raise AttributeError(name)

    async def _caller(self, operation_id, vars=None, **options):
        op = self._operations[operation_id]
        if vars is None:
            vars = {}

        data_reader = options.pop('data_reader', None)
        endpoint = op['endpoint'].format(**vars)
        verb = op['verb']
        options.update(op)
        resp_options = options.get('response', {})
        req_options = options.get('request', {})

        func = getattr(self.session, verb.lower())
        extra = {}
        extra['data'] = req_options.get('body') or req_options.get('data')
        extra['headers'] = req_options.get('headers')
        extra['params'] = req_options.get('params')

        async with func(endpoint, **extra) as resp:
            return await self._check_response(resp, resp_options, data_reader,
                                              options)

    async def _check_response(self, resp, resp_options, data_reader, options):
        check = options.get('check', True)
        status = resp.status

        # provided by the scenario (maybe should put it in responses)
        if check and 'status' in resp_options:
            wanted = int(resp_options['status'])
            if status != wanted:
                print("Bad Status code on %r" % options['endpoint'])
                print("Wanted %d, Got %d" % (wanted, status))
                body = await resp.text()
                raise AssertionError(body)

        if check and 'headers' in resp_options:
            for name, expected in resp_options['headers'].items():
                got = resp.headers.get(name)
                if got != expected:
                    print('Bad value for header %s' % name)
                    print('Got %r, expected %r' % (got, expected))
                    body = await resp.text()
                    raise AssertionError(body)

        # provided by swagger
        elif check:
            if 'default' not in options['responses']:
                # default means the status can be anything
                # so we're skipping this test
                # Note that in the future if we do more than asserting
                # the status code, we will nee to iterate over the responses
                # options even when default is present
                statuses = [int(st) for st in options['responses'].keys()]
                if status not in statuses:
                    print("Bad Status code on %r" % options['endpoint'])
                    statuses = ' or '.join(['%d' % s for s in statuses])
                    print("Wanted %s, Got %d" % (statuses, status))
                    body = await resp.text()
                    raise AssertionError(body)

        # extracting variables if needed
        vars = resp_options.get('vars')
        if vars and data_reader:
            json_data = await resp.json()

            for varname, data in vars.items():
                default = data.get('default')
                query = data['query']
                value = json_data.get(query, default)
                data_reader(varname, value)

        return resp

    def _get_operations(self):
        ops = {}
        for path, spec in self.spec['paths'].items():
            endpoint = urlunparse((self.scheme, self.host, path, '', '', ''))
            for verb, options in spec.items():
                if verb == 'parameters':
                    continue
                verb = verb.upper()
                options['verb'] = verb.upper()
                options['endpoint'] = endpoint
                ops[options['operationId']] = options
        return ops



from smwogger.cli import console


_VARS = {'prod': 'firefox',
         'ver': 43,
         'channel': 'release',
         'dist': 'default',
         'distver': 'default'}


async def scenario(api, args):

    with console('Getting heartbeat', verbose=args.verbose):
        resp = await api.getHeartbeat()

    assert resp.status == 200

    with console('Playing with the cohorts', verbose=args.verbose):
        vars = {'locale': 'en-US',
                'territory': 'US'}
        vars.update(_VARS)
        resp = await api.addUserToCohort(vars=vars)
        vars['cohort'] = 'default'
        resp = await api.returnCohortSettings(vars)



import argparse
import logging
from functools import partial
import imp
import asyncio

from smwogger import logger
from smwogger.cli import console
from smwogger.api import API
from smwogger.smoketest import SmokeTest


def get_runner(api, url, test_url=None, verbose=0, stream=None):
    if test_url and test_url.endswith('.py'):
        script = imp.load_source('script', test_url)
        return partial(script.scenario, api)
    else:
        return SmokeTest(api, test_url)


DONE = object()


async def log(queue):
    while True:
        try:
            item = queue.get_nowait()
        except RuntimeError:
            break
        except asyncio.QueueEmpty:
            await asyncio.sleep(.2)
        else:
            if item == DONE:
                return
            else:
                logger.info(item)


def main():
    parser = argparse.ArgumentParser(
        description='Smwogger. Smoke Tester.')

    parser.add_argument('url', help='Swagger URL or file')
    parser.add_argument('--test', help='Test URL or file',
                        default=None)
    parser.add_argument('-v', '--verbose', action='count', default=0,
                        help=('Verbosity level. -v will display '
                              'tracebacks. -vv requests and responses.'))

    args = parser.parse_args()
    url = args.url

    if args.verbose > 0:
        logger.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)
        formatter = logging.Formatter('%(message)s')
        ch.setFormatter(formatter)
        logger.addHandler(ch)
        logger.propagate = False

    loop = asyncio.get_event_loop()
    stream = asyncio.Queue()
    api = API(url, verbose=args.verbose, stream=stream)

    try:
        with console("Scanning spec", verbose=args.verbose):
            runner = get_runner(api, url, test_url=args.test,
                                verbose=args.verbose,
                                stream=stream)

        coros = []
        if isinstance(runner, SmokeTest):
            spec = runner.api.spec
            if args.verbose > 1:
                print()
                print("\t\tThis is project %r" % spec['info']['title'])
                print("\t\t%s" % spec['info']['description'])
                print("\t\tVersion %s" % spec['info']['version'])
                print()
                print()
                print('Running Scenario from x-smoke-test')

            async def _scenario():
                for index, (oid, options) in enumerate(runner.scenario()):
                    with console('%d:%s' % (index + 1, oid),
                                 verbose=args.verbose):
                        try:
                            await runner(oid, **options)
                        except Exception:
                            raise
                await stream.put(DONE)

            coros.append(_scenario())
        else:
            if args.verbose > 1:
                print('Running Python Scenario')

            async def single_run():
                await runner(args)
                await stream.put(DONE)

            coros.append(single_run())

        coros.append(log(stream))
        loop.run_until_complete(asyncio.gather(*coros))
    finally:
        api.close()
        loop.close()

    if args.verbose == 0:
        print()



from bottle import response, Bottle, request
import os
import yaml


HERE = os.path.dirname(__file__)
SPEC = os.path.join(HERE, 'absearch.yaml')
app = application = Bottle()


@app.route('/api.yaml')
def swagger_yaml():
    with open(SPEC) as f:
        data = f.read()
    response.set_header('Content-Type', 'text/yaml')
    return data


@app.route('/api.json')
def swagger():
    with open(SPEC) as f:
        data = yaml.load(f.read())
    return data


@app.route('/1/<path:path>')
def someapi(*args, **kw):
    return {}


@app.route('/1/badstatus')
def badstatus():
    return {'result': 'OK'}


@app.route('/1')
def root():
    return {'result': 'OK'}


@app.route('/1/__heartbeat__')
def hb():
    return {'result': 'OK', 'headers': dict(request.headers)}


if __name__ == '__main__':
    app.run(port=8888)



import os
import urllib.request
import mimetypes
import json
import yaml


_JSON_TYPES = ('application/vnd.api+json', 'application/json')
_YAML_TYPES = ('application/x-yaml', 'text/yaml')

if '.yaml' not in mimetypes.types_map:
    mimetypes.types_map['.yaml'] = 'application/x-yaml'


def _decoder(mime):
    if mime in _YAML_TYPES:
        return yaml.load
    # we'll just try json
    return json.loads


def get_content(url):
    if os.path.exists(url):
        mime = mimetypes.guess_type(url)[0]
        with open(url) as f:
            return _decoder(mime)(f.read())
    else:
        resp = urllib.request.urlopen(url)
        content_type = resp.headers.get('Content-Type', 'application/json')
        content = resp.read()
        content = str(content, 'utf8')
        return _decoder(content_type)(content)



import logging


logger = logging.getLogger('smwogger')    # NOQA
logging.basicConfig(level=logging.WARNING)

try:
    from smwogger.api import API          # NOQA
except ImportError:
    # first import (setuptools)
    pass                                  # NOQA



import unittest
import os

from smwogger.api import API
from smwogger.tests.support import coserver, async_test


HERE = os.path.dirname(__file__)
SPEC = os.path.join(HERE, 'absearch.yaml')


class TestAPI(unittest.TestCase):

    @async_test
    async def test_names_regular_enter_exit(self, loop):
        with API(SPEC, verbose=True, loop=loop) as api:
            with coserver():
                await api.getHeartbeat()

            for attr in ('getHeartbeat', 'addUserToCohort',
                         'returnCohortSettings'):
                self.assertTrue(hasattr(api, attr))

    @async_test
    async def test_names(self, loop):
        async with API(SPEC, verbose=True, loop=loop) as api:
            with coserver():
                await api.getHeartbeat()

            for attr in ('getHeartbeat', 'addUserToCohort',
                         'returnCohortSettings'):
                self.assertTrue(hasattr(api, attr))

    @async_test
    async def test_default(self, loop):
        async with API(SPEC, loop=loop) as api:
            with coserver():
                await api.getDefault()

    @async_test
    async def test_read_spec_from_url(self, loop):
        headers = {'Something': 'here'}

        with coserver():
            async with API('http://localhost:8888/api.yaml', verbose=True,
                           loop=loop) as api:
                await api.getDefault()

            async with API('http://localhost:8888/api.json', verbose=True,
                           loop=loop) as api:
                await api.getDefault()
                res = await api.getHeartbeat(request={'headers': headers})

        data = await res.json()
        echoed_headers = data['headers']
        self.assertEqual(echoed_headers['Something'], 'here')

    @async_test
    async def test_bad_method(self, loop):
        async with API(SPEC, verbose=True, loop=loop) as api:
            try:
                await api.iDontExist()
                raise AssertionError("WAT")
            except AttributeError:
                pass

    @async_test
    async def test_bad_headers(self, loop):
        headers = {'Content-Type': 'nevergonnahappen'}

        with coserver():
            async with API('http://localhost:8888/api.yaml', verbose=True,
                           loop=loop) as api:
                try:
                    await api.getHeartbeat(response={'headers': headers})
                    raise AssertionError("WAT")
                except AssertionError:
                    pass

    @async_test
    async def test_bad_status2(self, loop):
        with coserver():
            async with API('http://localhost:8888/api.yaml', verbose=True,
                           loop=loop) as api:
                try:
                    await api.getHeartbeat(response={'status': 209})
                    raise AssertionError("WAT")
                except AssertionError:
                    pass

    @async_test
    async def test_bad_status(self, loop):
        with coserver():
            async with API('http://localhost:8888/api.yaml', verbose=True,
                           loop=loop) as api:
                try:
                    await api.getBadStatus()
                    raise AssertionError("WAT")
                except AssertionError:
                    pass

    @async_test
    async def test_get_ops(self, loop):
        wanted = ['addUserToCohort', 'getBadStatus', 'getDefault',
                  'getHeartbeat', 'getInfo', 'getRoot',
                  'returnCohortSettings']

        async with API(SPEC, verbose=True, loop=loop) as api:
            self.assertEqual(api.operations, wanted)



from smwogger.util import get_content


class SmokeTest(object):
    def __init__(self, api, test_url=None):
        if test_url:
            self._content = get_content(test_url)['x-smoke-test']
        else:
            self._content = api.spec['x-smoke-test']
        self.api = api
        self._vars = {}

    def __call__(self, operation_id, **options):
        extra = options.get('request', {}).get('path', {})
        vars = dict(self._content.get('path', {}))
        vars.update(self._vars)
        vars.update(extra)
        options['data_reader'] = self.set_var
        return self.api._caller(operation_id, vars=vars, **options)

    def scenario(self):
        return [list(step.items())[0] for step in
                self._content.get('scenario', [])]

    def set_var(self, name, value):
        self._vars[name] = value

    def get_var(self, name):
        return self._vars[name]



import os
import unittest

from smwogger.tests.support import set_args, coserver, dedicatedloop
from smwogger.main import main


_SCENARIO = os.path.join(os.path.dirname(__file__), 'scenario.py')
WANTED = """\
Scanning spec... \x1b[92mOK\x1b[0m

\t\tThis is project 'ABSearch Server'
\t\tlightweight a/b testing tool for search options
\t\tVersion 0.3.0


Running Scenario from x-smoke-test
1:getHeartbeat... \x1b[92mOK\x1b[0m
2:addUserToCohort... \x1b[92mOK\x1b[0m
3:returnCohortSettings... \x1b[92mOK\x1b[0m"""


WANTED2 = """\
Scanning spec... \x1b[92mOK\x1b[0m
Getting heartbeat... \x1b[92mOK\x1b[0m
Playing with the cohorts... \x1b[92mOK\x1b[0m"""


WANTED3 = """\
Scanning spec... \x1b[92mOK\x1b[0m
Running Python Scenario
Getting heartbeat... \x1b[92mOK\x1b[0m
Playing with the cohorts... \x1b[92mOK\x1b[0m"""


class TestMain(unittest.TestCase):

    @dedicatedloop
    def test_main(self):

        options = ('smwogger', 'http://localhost:8888/api.json', '--verbose',
                   '--verbose')

        with coserver(), set_args(*options) as out:
            try:
                main()
            except SystemExit:
                pass
            except Exception:
                pass
        stdout = out[0].read().strip()
        self.assertEqual(stdout, WANTED)
        stderr = out[1].read().strip()
        self.assertTrue("Content-Type: application/json" in stderr)

    @dedicatedloop
    def test_scenario(self):
        options = ('smwogger', '-v', '--test', _SCENARIO,
                   'http://localhost:8888/api.json')

        with coserver(), set_args(*options) as out:
            try:
                main()
            except SystemExit:
                pass

        stdout = out[0].read().strip()
        self.assertEqual(stdout, WANTED2)

    @dedicatedloop
    def test_scenario_verbose(self):
        options = ('smwogger', '-vv', '--test', _SCENARIO,
                   'http://localhost:8888/api.json')

        with coserver(), set_args(*options) as out:
            try:
                main()
            except SystemExit:
                pass

        stdout = out[0].read().strip()
        self.assertEqual(stdout, WANTED3)



import asyncio
import json
from uuid import uuid4
from smwogger import API


async def print_operations(q):

    name = "tarekounet-%s" % (str(uuid4()))
    data = {}

    def reader(name, value):
        data[name] = value

    async with API('http://petstore.swagger.io/v2/swagger.json', verbose=2,
                   stream=q) as api:
        # let's add a new pet
        pet = {"name": name,
               "photoUrls": ["string"],
               "status": "available"}

        pet = json.dumps(pet)
        headers = {'Content-Type': 'application/json',
                   'api_key': 'special-key'}
        res = await api.addPet(check=False,
                               request={'data': pet, 'headers': headers},
                               response={'vars': {'id': {'query': 'id'}}},
                               data_reader=reader)

        result = await res.json()
        vars = {'petId': data['id']}
        res = await api.getPetById(vars=vars, data_reader=reader,
                                   request={'headers': headers})
        result = await res.json()
        print(result)


loop = asyncio.get_event_loop()
q = asyncio.Queue()

try:
    loop.run_until_complete(print_operations(q))
finally:
    while not q.empty():
        print(q.get_nowait())
    loop.close()



import sys
from setuptools import setup, find_packages


with open('requirements.txt') as f:
    deps = [dep for dep in f.read().split('\n') if dep.strip() != '']
    install_requires = deps

with open('README.rst') as f:
    _DESC = f.read() + '\n\n'

with open('CONTRIBUTORS.rst') as f:
    _DESC += f.read() + '\n\n'

with open('CHANGELOG.rst') as f:
    _DESC += f.read()


setup(name='smwogger',
      version="1.0",
      url="https://github.com/tarekziade/smwogger",
      packages=find_packages(),
      author="Tarek Ziade",
      author_email="tarek@ziade.org",
      description="Smoke Test tool",
      long_description=_DESC,
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      entry_points="""
      [console_scripts]
      smwogger = smwogger.main:main
      """)



import asyncio
import sys
import signal
import os
import multiprocessing
import time
from contextlib import contextmanager
from http.client import HTTPConnection
import functools
from io import StringIO


def run_server(port=8888):
    """Running in a subprocess to avoid any interference
    """
    def _run():
        import sys
        import io

        sys.stderr = sys.stdout = io.StringIO()

        from smwogger.tests.service import application
        import signal

        def _shutdown(*args, **kw):
            application.close()
            sys.exit(0)

        signal.signal(signal.SIGTERM, _shutdown)
        signal.signal(signal.SIGINT, _shutdown)
        application.run(host='localhost', port=port)

    p = multiprocessing.Process(target=_run)
    p.start()
    start = time.time()
    connected = False

    while time.time() - start < 5 and not connected:
        try:
            conn = HTTPConnection('localhost', port)
            conn.request("GET", "/")
            conn.getresponse()
            connected = True
        except Exception:
            time.sleep(.1)
    if not connected:
        os.kill(p.pid, signal.SIGTERM)
        p.join(timeout=1.)
        raise OSError('Could not connect to coserver')
    return p


_CO = {'clients': 0, 'server': None}


@contextmanager
def coserver(port=8888):
    if _CO['clients'] == 0:
        _CO['server'] = run_server(port)

    _CO['clients'] += 1
    try:
        yield
    finally:
        _CO['clients'] -= 1
        if _CO['clients'] == 0:
            os.kill(_CO['server'].pid, signal.SIGTERM)
            _CO['server'].join(timeout=1.)
            _CO['server'] = None


@contextmanager
def set_args(*args):
    old = list(sys.argv)
    sys.argv[:] = args
    oldout, olderr = sys.stdout, sys.stderr
    sys.stdout, sys.stderr = StringIO(), StringIO()
    try:
        yield sys.stdout, sys.stderr
    finally:
        sys.stdout.seek(0)
        sys.stderr.seek(0)
        sys.argv[:] = old
        sys.stdout, sys.stderr = oldout, olderr


def async_test(func):
    @functools.wraps(func)
    def _async_test(*args, **kw):
        cofunc = asyncio.coroutine(func)
        oldloop = asyncio.get_event_loop()
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.set_debug(True)
        kw['loop'] = loop
        try:
            loop.run_until_complete(cofunc(*args, **kw))
        finally:
            loop.stop()
            loop.close()
            asyncio.set_event_loop(oldloop)
    return _async_test


def dedicatedloop(func):
    @functools.wraps(func)
    def _loop(*args, **kw):
        old_loop = asyncio.get_event_loop()
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            return func(*args, **kw)
        finally:
            asyncio.set_event_loop(old_loop)
    return _loop



import sys
from contextlib import contextmanager


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


def stdout(msg):
    sys.stdout.write(msg)
    sys.stdout.flush()


def eol():
    stdout('\n')


@contextmanager
def console(description, success='OK', fail='FAIL', verbose=1):
    if verbose > 0:
        stdout(description + '... ')
    try:
        yield
        if verbose > 0:
            stdout(bcolors.OKGREEN + success + bcolors.ENDC)
    except Exception:
        if verbose > 0:
            stdout(bcolors.FAIL + fail + bcolors.ENDC)
        raise
    if verbose == 0:
        stdout('.')
    else:
        eol()
